#
# Automated Testing Framework (atf)
#
# Copyright (c) 2007, 2008 The NetBSD Foundation, Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE NETBSD FOUNDATION, INC. AND
# CONTRIBUTORS ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES,
# INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
# IN NO EVENT SHALL THE FOUNDATION OR CONTRIBUTORS BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
# GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
# IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
# OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
# IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#

run_tests()
{
    h=${1} what=${2} status=${3} code=${4}

    atf_check -s eq:${code} -o ignore -e ignore -x \
              "${h} -s $(atf_get_srcdir) -r3 -v 'cleanup=no' \
               -v 'tmpfile=$(pwd)/tmpfile' \
               cleanup_${what} 3>resout"
    atf_check -s eq:0 -o ignore -e empty \
              grep "cleanup_${what}, ${status}" resout
    atf_check -s eq:0 -o empty -e empty test -f tmpfile
    atf_check -s eq:0 -o empty -e empty rm tmpfile

    atf_check -s eq:${code} -o ignore -e ignore -x \
              "${h} -s $(atf_get_srcdir) -r3 -v 'cleanup=yes' \
               -v 'tmpfile=$(pwd)/tmpfile' \
               cleanup_${what} 3>resout"
    atf_check -s eq:0 -o ignore -e empty \
              grep "cleanup_${what}, ${status}" resout
    atf_check -s eq:1 -o empty -e empty test -f tmpfile
}

atf_test_case hook
hook_head()
{
    atf_set "descr" "Tests that the cleanup hook works"
}
hook_body()
{
    for h in $(get_helpers); do
        run_tests ${h} pass passed 0
        run_tests ${h} fail failed 1
        run_tests ${h} skip skipped 0
    done
}

atf_test_case curdir
curdir_head()
{
    atf_set "descr" "Tests that the cleanup hook is executed in the same" \
                    "directory as the test case so that they can share" \
                    "information"
}
curdir_body()
{
    for h in $(get_helpers); do
        atf_check -s eq:0 -o save:stdout -e ignore -x \
                  "${h} -s $(atf_get_srcdir) -r3 cleanup_curdir 3>resout"
        atf_check -s eq:0 -o ignore -e empty grep "Old value: 1234" stdout
    done
}

atf_test_case on_signal
on_signal_head()
{
    atf_set "descr" "Tests that the cleanup routines are executed when" \
                    "test cases receive a signal during execution"
}
on_signal_body()
{
    for h in $(get_helpers); do
        case ${h} in
            *h_sh*)
                # XXX Broken for now.
                continue
                ;;
        esac

        ${h} -s $(atf_get_srcdir) -r3 -v "tmpfile=$(pwd)/tmpfile" \
            cleanup_sigterm 3>resout
        atf_check -s eq:0 -o ignore -e empty \
                  grep 'cleanup_sigterm, failed' resout
        atf_check -s eq:1 -o empty -e empty test -f tmpfile
        atf_check -s eq:1 -o empty -e empty test -f tmpfile.no
    done
}

atf_test_case fork
fork_head()
{
    atf_set "descr" "Tests that the cleanup routines are executed in" \
                    "a subprocess"
}
fork_body()
{
    for h in $(get_helpers); do
        ${h} -s $(atf_get_srcdir) -r3 -v "tmpfile=$(pwd)/tmpfile" \
            cleanup_fork 3>resout
        atf_check -s eq:0 -o ignore -e empty \
                  grep 'cleanup_fork, passed' resout
    done
}

atf_init_test_cases()
{
    atf_add_test_case hook
    atf_add_test_case curdir
    atf_add_test_case on_signal
    atf_add_test_case fork
}

# vim: syntax=sh:expandtab:shiftwidth=4:softtabstop=4
